// Copyright 2024 The Chromium Authors
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// GENERATED FROM THE API DEFINITION IN
//   extensions/common/api/extension_types.json
// by tools/json_schema_compiler.
// DO NOT EDIT.

#include "extensions/common/api/extension_types.h"

#include <memory>
#include <ostream>
#include <string>
#include <utility>
#include <vector>

#include "base/check.h"
#include "base/check_op.h"
#include "base/notreached.h"
#include "base/strings/string_number_conversions.h"
#include "base/strings/utf_string_conversions.h"
#include "base/values.h"
#include "tools/json_schema_compiler/util.h"

using base::UTF8ToUTF16;

namespace extensions {
namespace api {
namespace extension_types {
//
// Types
//

const char* ToString(ImageFormat enum_param) {
  switch (enum_param) {
    case IMAGE_FORMAT_JPEG:
      return "jpeg";
    case IMAGE_FORMAT_PNG:
      return "png";
    case IMAGE_FORMAT_NONE:
      return "";
  }
  NOTREACHED();
  return "";
}

ImageFormat ParseImageFormat(const std::string& enum_string) {
  if (enum_string == "jpeg")
    return IMAGE_FORMAT_JPEG;
  if (enum_string == "png")
    return IMAGE_FORMAT_PNG;
  return IMAGE_FORMAT_NONE;
}


ImageDetails::ImageDetails()
: format(IMAGE_FORMAT_NONE) {}

ImageDetails::~ImageDetails() = default;
ImageDetails::ImageDetails(ImageDetails&& rhs) = default;
ImageDetails& ImageDetails::operator=(ImageDetails&& rhs) = default;
// static
bool ImageDetails::Populate(
    const base::Value& value, ImageDetails* out) {
  if (!value.is_dict()) {
    return false;
  }
  const base::Value::Dict& dict = value.GetDict();
  out->format = IMAGE_FORMAT_NONE;
  const base::Value* format_value = dict.Find("format");
  if (format_value) {
    {
      const std::string* image_format_as_string = (*format_value).GetIfString();
      if (!image_format_as_string) {
        return false;
      }
      out->format = ParseImageFormat(*image_format_as_string);
      if (out->format == IMAGE_FORMAT_NONE) {
        return false;
      }
    }
    } else {
    out->format = IMAGE_FORMAT_NONE;
  }

  const base::Value* quality_value = dict.Find("quality");
  if (quality_value) {
    {
      auto temp = (*quality_value).GetIfInt();
      if (!temp.has_value()) {
        out->quality = absl::nullopt;
        return false;
      }
      out->quality = *temp;
    }
  }

  return true;
}

// static
std::unique_ptr<ImageDetails> ImageDetails::FromValue(const base::Value& value) {
  auto out = std::make_unique<ImageDetails>();
  bool result = Populate(value, out.get());
  if (!result)
    return nullptr;
  return out;
}

base::Value::Dict ImageDetails::ToValue() const {
  base::Value::Dict to_value_result;

  if (this->format != IMAGE_FORMAT_NONE) {
    to_value_result.Set("format", extension_types::ToString(this->format));

  }
  if (this->quality) {
    to_value_result.Set("quality", *this->quality);

  }

  return to_value_result;
}


const char* ToString(RunAt enum_param) {
  switch (enum_param) {
    case RUN_AT_DOCUMENT_START:
      return "document_start";
    case RUN_AT_DOCUMENT_END:
      return "document_end";
    case RUN_AT_DOCUMENT_IDLE:
      return "document_idle";
    case RUN_AT_NONE:
      return "";
  }
  NOTREACHED();
  return "";
}

RunAt ParseRunAt(const std::string& enum_string) {
  if (enum_string == "document_start")
    return RUN_AT_DOCUMENT_START;
  if (enum_string == "document_end")
    return RUN_AT_DOCUMENT_END;
  if (enum_string == "document_idle")
    return RUN_AT_DOCUMENT_IDLE;
  return RUN_AT_NONE;
}


const char* ToString(CSSOrigin enum_param) {
  switch (enum_param) {
    case CSS_ORIGIN_AUTHOR:
      return "author";
    case CSS_ORIGIN_USER:
      return "user";
    case CSS_ORIGIN_NONE:
      return "";
  }
  NOTREACHED();
  return "";
}

CSSOrigin ParseCSSOrigin(const std::string& enum_string) {
  if (enum_string == "author")
    return CSS_ORIGIN_AUTHOR;
  if (enum_string == "user")
    return CSS_ORIGIN_USER;
  return CSS_ORIGIN_NONE;
}


InjectDetails::InjectDetails()
: run_at(RUN_AT_NONE),
css_origin(CSS_ORIGIN_NONE) {}

InjectDetails::~InjectDetails() = default;
InjectDetails::InjectDetails(InjectDetails&& rhs) = default;
InjectDetails& InjectDetails::operator=(InjectDetails&& rhs) = default;
// static
bool InjectDetails::Populate(
    const base::Value& value, InjectDetails* out) {
  if (!value.is_dict()) {
    return false;
  }
  const base::Value::Dict& dict = value.GetDict();
  out->run_at = RUN_AT_NONE;
  out->css_origin = CSS_ORIGIN_NONE;
  const base::Value* code_value = dict.Find("code");
  if (code_value) {
    {
      auto* temp = (*code_value).GetIfString();
      if (!temp) {
        out->code = absl::nullopt;
        return false;
      }
      out->code = *temp;
    }
  }

  const base::Value* file_value = dict.Find("file");
  if (file_value) {
    {
      auto* temp = (*file_value).GetIfString();
      if (!temp) {
        out->file = absl::nullopt;
        return false;
      }
      out->file = *temp;
    }
  }

  const base::Value* all_frames_value = dict.Find("allFrames");
  if (all_frames_value) {
    {
      auto temp = (*all_frames_value).GetIfBool();
      if (!temp.has_value()) {
        out->all_frames = absl::nullopt;
        return false;
      }
      out->all_frames = *temp;
    }
  }

  const base::Value* frame_id_value = dict.Find("frameId");
  if (frame_id_value) {
    {
      auto temp = (*frame_id_value).GetIfInt();
      if (!temp.has_value()) {
        out->frame_id = absl::nullopt;
        return false;
      }
      out->frame_id = *temp;
    }
  }

  const base::Value* match_about_blank_value = dict.Find("matchAboutBlank");
  if (match_about_blank_value) {
    {
      auto temp = (*match_about_blank_value).GetIfBool();
      if (!temp.has_value()) {
        out->match_about_blank = absl::nullopt;
        return false;
      }
      out->match_about_blank = *temp;
    }
  }

  const base::Value* run_at_value = dict.Find("runAt");
  if (run_at_value) {
    {
      const std::string* run_at_as_string = (*run_at_value).GetIfString();
      if (!run_at_as_string) {
        return false;
      }
      out->run_at = ParseRunAt(*run_at_as_string);
      if (out->run_at == RUN_AT_NONE) {
        return false;
      }
    }
    } else {
    out->run_at = RUN_AT_NONE;
  }

  const base::Value* css_origin_value = dict.Find("cssOrigin");
  if (css_origin_value) {
    {
      const std::string* css_origin_as_string = (*css_origin_value).GetIfString();
      if (!css_origin_as_string) {
        return false;
      }
      out->css_origin = ParseCSSOrigin(*css_origin_as_string);
      if (out->css_origin == CSS_ORIGIN_NONE) {
        return false;
      }
    }
    } else {
    out->css_origin = CSS_ORIGIN_NONE;
  }

  return true;
}

// static
std::unique_ptr<InjectDetails> InjectDetails::FromValue(const base::Value& value) {
  auto out = std::make_unique<InjectDetails>();
  bool result = Populate(value, out.get());
  if (!result)
    return nullptr;
  return out;
}

base::Value::Dict InjectDetails::ToValue() const {
  base::Value::Dict to_value_result;

  if (this->code) {
    to_value_result.Set("code", *this->code);

  }
  if (this->file) {
    to_value_result.Set("file", *this->file);

  }
  if (this->all_frames) {
    to_value_result.Set("allFrames", *this->all_frames);

  }
  if (this->frame_id) {
    to_value_result.Set("frameId", *this->frame_id);

  }
  if (this->match_about_blank) {
    to_value_result.Set("matchAboutBlank", *this->match_about_blank);

  }
  if (this->run_at != RUN_AT_NONE) {
    to_value_result.Set("runAt", extension_types::ToString(this->run_at));

  }
  if (this->css_origin != CSS_ORIGIN_NONE) {
    to_value_result.Set("cssOrigin", extension_types::ToString(this->css_origin));

  }

  return to_value_result;
}


DeleteInjectionDetails::DeleteInjectionDetails()
: css_origin(CSS_ORIGIN_NONE) {}

DeleteInjectionDetails::~DeleteInjectionDetails() = default;
DeleteInjectionDetails::DeleteInjectionDetails(DeleteInjectionDetails&& rhs) = default;
DeleteInjectionDetails& DeleteInjectionDetails::operator=(DeleteInjectionDetails&& rhs) = default;
// static
bool DeleteInjectionDetails::Populate(
    const base::Value& value, DeleteInjectionDetails* out) {
  if (!value.is_dict()) {
    return false;
  }
  const base::Value::Dict& dict = value.GetDict();
  out->css_origin = CSS_ORIGIN_NONE;
  const base::Value* code_value = dict.Find("code");
  if (code_value) {
    {
      auto* temp = (*code_value).GetIfString();
      if (!temp) {
        out->code = absl::nullopt;
        return false;
      }
      out->code = *temp;
    }
  }

  const base::Value* file_value = dict.Find("file");
  if (file_value) {
    {
      auto* temp = (*file_value).GetIfString();
      if (!temp) {
        out->file = absl::nullopt;
        return false;
      }
      out->file = *temp;
    }
  }

  const base::Value* all_frames_value = dict.Find("allFrames");
  if (all_frames_value) {
    {
      auto temp = (*all_frames_value).GetIfBool();
      if (!temp.has_value()) {
        out->all_frames = absl::nullopt;
        return false;
      }
      out->all_frames = *temp;
    }
  }

  const base::Value* frame_id_value = dict.Find("frameId");
  if (frame_id_value) {
    {
      auto temp = (*frame_id_value).GetIfInt();
      if (!temp.has_value()) {
        out->frame_id = absl::nullopt;
        return false;
      }
      out->frame_id = *temp;
    }
  }

  const base::Value* match_about_blank_value = dict.Find("matchAboutBlank");
  if (match_about_blank_value) {
    {
      auto temp = (*match_about_blank_value).GetIfBool();
      if (!temp.has_value()) {
        out->match_about_blank = absl::nullopt;
        return false;
      }
      out->match_about_blank = *temp;
    }
  }

  const base::Value* css_origin_value = dict.Find("cssOrigin");
  if (css_origin_value) {
    {
      const std::string* css_origin_as_string = (*css_origin_value).GetIfString();
      if (!css_origin_as_string) {
        return false;
      }
      out->css_origin = ParseCSSOrigin(*css_origin_as_string);
      if (out->css_origin == CSS_ORIGIN_NONE) {
        return false;
      }
    }
    } else {
    out->css_origin = CSS_ORIGIN_NONE;
  }

  return true;
}

// static
std::unique_ptr<DeleteInjectionDetails> DeleteInjectionDetails::FromValue(const base::Value& value) {
  auto out = std::make_unique<DeleteInjectionDetails>();
  bool result = Populate(value, out.get());
  if (!result)
    return nullptr;
  return out;
}

base::Value::Dict DeleteInjectionDetails::ToValue() const {
  base::Value::Dict to_value_result;

  if (this->code) {
    to_value_result.Set("code", *this->code);

  }
  if (this->file) {
    to_value_result.Set("file", *this->file);

  }
  if (this->all_frames) {
    to_value_result.Set("allFrames", *this->all_frames);

  }
  if (this->frame_id) {
    to_value_result.Set("frameId", *this->frame_id);

  }
  if (this->match_about_blank) {
    to_value_result.Set("matchAboutBlank", *this->match_about_blank);

  }
  if (this->css_origin != CSS_ORIGIN_NONE) {
    to_value_result.Set("cssOrigin", extension_types::ToString(this->css_origin));

  }

  return to_value_result;
}


const char* ToString(FrameType enum_param) {
  switch (enum_param) {
    case FRAME_TYPE_OUTERMOST_FRAME:
      return "outermost_frame";
    case FRAME_TYPE_FENCED_FRAME:
      return "fenced_frame";
    case FRAME_TYPE_SUB_FRAME:
      return "sub_frame";
    case FRAME_TYPE_NONE:
      return "";
  }
  NOTREACHED();
  return "";
}

FrameType ParseFrameType(const std::string& enum_string) {
  if (enum_string == "outermost_frame")
    return FRAME_TYPE_OUTERMOST_FRAME;
  if (enum_string == "fenced_frame")
    return FRAME_TYPE_FENCED_FRAME;
  if (enum_string == "sub_frame")
    return FRAME_TYPE_SUB_FRAME;
  return FRAME_TYPE_NONE;
}


const char* ToString(DocumentLifecycle enum_param) {
  switch (enum_param) {
    case DOCUMENT_LIFECYCLE_PRERENDER:
      return "prerender";
    case DOCUMENT_LIFECYCLE_ACTIVE:
      return "active";
    case DOCUMENT_LIFECYCLE_CACHED:
      return "cached";
    case DOCUMENT_LIFECYCLE_PENDING_DELETION:
      return "pending_deletion";
    case DOCUMENT_LIFECYCLE_NONE:
      return "";
  }
  NOTREACHED();
  return "";
}

DocumentLifecycle ParseDocumentLifecycle(const std::string& enum_string) {
  if (enum_string == "prerender")
    return DOCUMENT_LIFECYCLE_PRERENDER;
  if (enum_string == "active")
    return DOCUMENT_LIFECYCLE_ACTIVE;
  if (enum_string == "cached")
    return DOCUMENT_LIFECYCLE_CACHED;
  if (enum_string == "pending_deletion")
    return DOCUMENT_LIFECYCLE_PENDING_DELETION;
  return DOCUMENT_LIFECYCLE_NONE;
}


const char* ToString(ExecutionWorld enum_param) {
  switch (enum_param) {
    case EXECUTION_WORLD_ISOLATED:
      return "ISOLATED";
    case EXECUTION_WORLD_MAIN:
      return "MAIN";
    case EXECUTION_WORLD_NONE:
      return "";
  }
  NOTREACHED();
  return "";
}

ExecutionWorld ParseExecutionWorld(const std::string& enum_string) {
  if (enum_string == "ISOLATED")
    return EXECUTION_WORLD_ISOLATED;
  if (enum_string == "MAIN")
    return EXECUTION_WORLD_MAIN;
  return EXECUTION_WORLD_NONE;
}



}  // namespace extension_types
}  // namespace api
}  // namespace extensions

