// Copyright 2024 The Chromium Authors
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// GENERATED FROM THE API DEFINITION IN
//   extensions/common/api/offscreen.idl
// by tools/json_schema_compiler.
// DO NOT EDIT.

#include "extensions/common/api/offscreen.h"

#include <memory>
#include <ostream>
#include <string>
#include <utility>
#include <vector>

#include "base/check.h"
#include "base/check_op.h"
#include "base/notreached.h"
#include "base/strings/string_number_conversions.h"
#include "base/strings/utf_string_conversions.h"
#include "base/values.h"
#include "tools/json_schema_compiler/util.h"

using base::UTF8ToUTF16;

namespace extensions {
namespace api {
namespace offscreen {
//
// Types
//

const char* ToString(Reason enum_param) {
  switch (enum_param) {
    case REASON_TESTING:
      return "TESTING";
    case REASON_AUDIO_PLAYBACK:
      return "AUDIO_PLAYBACK";
    case REASON_IFRAME_SCRIPTING:
      return "IFRAME_SCRIPTING";
    case REASON_DOM_SCRAPING:
      return "DOM_SCRAPING";
    case REASON_BLOBS:
      return "BLOBS";
    case REASON_DOM_PARSER:
      return "DOM_PARSER";
    case REASON_USER_MEDIA:
      return "USER_MEDIA";
    case REASON_DISPLAY_MEDIA:
      return "DISPLAY_MEDIA";
    case REASON_WEB_RTC:
      return "WEB_RTC";
    case REASON_CLIPBOARD:
      return "CLIPBOARD";
    case REASON_NONE:
      return "";
  }
  NOTREACHED();
  return "";
}

Reason ParseReason(const std::string& enum_string) {
  if (enum_string == "TESTING")
    return REASON_TESTING;
  if (enum_string == "AUDIO_PLAYBACK")
    return REASON_AUDIO_PLAYBACK;
  if (enum_string == "IFRAME_SCRIPTING")
    return REASON_IFRAME_SCRIPTING;
  if (enum_string == "DOM_SCRAPING")
    return REASON_DOM_SCRAPING;
  if (enum_string == "BLOBS")
    return REASON_BLOBS;
  if (enum_string == "DOM_PARSER")
    return REASON_DOM_PARSER;
  if (enum_string == "USER_MEDIA")
    return REASON_USER_MEDIA;
  if (enum_string == "DISPLAY_MEDIA")
    return REASON_DISPLAY_MEDIA;
  if (enum_string == "WEB_RTC")
    return REASON_WEB_RTC;
  if (enum_string == "CLIPBOARD")
    return REASON_CLIPBOARD;
  return REASON_NONE;
}


CreateParameters::CreateParameters()
 {}

CreateParameters::~CreateParameters() = default;
CreateParameters::CreateParameters(CreateParameters&& rhs) = default;
CreateParameters& CreateParameters::operator=(CreateParameters&& rhs) = default;
// static
bool CreateParameters::Populate(
    const base::Value& value, CreateParameters* out) {
  if (!value.is_dict()) {
    return false;
  }
  const base::Value::Dict& dict = value.GetDict();
  const base::Value* reasons_value = dict.Find("reasons");
  if (!reasons_value) {
    return false;
  }
  {
    if (!(*reasons_value).is_list()) {
      return false;
    }
    else {
      for (const auto& it : ((*reasons_value)).GetList()) {
        Reason tmp;
        const std::string* reason_as_string = (it).GetIfString();
        if (!reason_as_string) {
          return false;
        }
        tmp = ParseReason(*reason_as_string);
        if (tmp == REASON_NONE) {
          return false;
        }
        out->reasons.push_back(tmp);
      }
    }
  }

  const base::Value* url_value = dict.Find("url");
  if (!url_value) {
    return false;
  }
  {
    auto* temp = (*url_value).GetIfString();
    if (!temp) {
      return false;
    }
    out->url = *temp;
  }

  const base::Value* justification_value = dict.Find("justification");
  if (!justification_value) {
    return false;
  }
  {
    auto* temp = (*justification_value).GetIfString();
    if (!temp) {
      return false;
    }
    out->justification = *temp;
  }

  return true;
}

// static
std::unique_ptr<CreateParameters> CreateParameters::FromValue(const base::Value& value) {
  auto out = std::make_unique<CreateParameters>();
  bool result = Populate(value, out.get());
  if (!result)
    return nullptr;
  return out;
}

base::Value::Dict CreateParameters::ToValue() const {
  base::Value::Dict to_value_result;

  {
    std::vector<std::string> reasons_list;
    for (const auto& it : (this->reasons)) {
    reasons_list.push_back(offscreen::ToString(it));
  }
  to_value_result.Set("reasons", json_schema_compiler::util::CreateValueFromArray(reasons_list));
  }

  to_value_result.Set("url", this->url);

  to_value_result.Set("justification", this->justification);


  return to_value_result;
}



//
// Functions
//

namespace CreateDocument {

Params::Params() = default;
Params::~Params() = default;

// static
std::unique_ptr<Params> Params::Create(const base::Value::List& args) {
  if (args.size() != 1) {
    return nullptr;
  }
  std::unique_ptr<Params> params(new Params());

  if (0 < args.size() &&
      !args[0].is_none()) {
    const base::Value& parameters_value = args[0];
    {
      if (!parameters_value.is_dict()) {
        return std::unique_ptr<Params>();
      }
      if (!CreateParameters::Populate(parameters_value, &params->parameters)) {
        return std::unique_ptr<Params>();
      }
    }
  }
  else {
    return std::unique_ptr<Params>();
  }

  return params;
}


base::Value::List Results::Create() {
  base::Value::List create_results;

  return create_results;
}
}  // namespace CreateDocument

namespace CloseDocument {

base::Value::List Results::Create() {
  base::Value::List create_results;

  return create_results;
}
}  // namespace CloseDocument

namespace HasDocument {

base::Value::List Results::Create(bool result) {
  base::Value::List create_results;
  create_results.reserve(1);
  create_results.Append(result);

  return create_results;
}
}  // namespace HasDocument

}  // namespace offscreen
}  // namespace api
}  // namespace extensions

