// Copyright 2024 The Chromium Authors
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// GENERATED FROM THE API DEFINITION IN
//   extensions/common/api/runtime.json
// by tools/json_schema_compiler.
// DO NOT EDIT.

#include "extensions/common/api/runtime.h"

#include <memory>
#include <ostream>
#include <string>
#include <utility>
#include <vector>

#include "base/check.h"
#include "base/check_op.h"
#include "base/notreached.h"
#include "base/strings/string_number_conversions.h"
#include "base/strings/utf_string_conversions.h"
#include "base/values.h"
#include "tools/json_schema_compiler/util.h"

using base::UTF8ToUTF16;

namespace extensions {
namespace api {
namespace runtime {
//
// Properties
//

//
// Types
//

const char* ToString(PlatformOs enum_param) {
  switch (enum_param) {
    case PLATFORM_OS_MAC:
      return "mac";
    case PLATFORM_OS_WIN:
      return "win";
    case PLATFORM_OS_ANDROID:
      return "android";
    case PLATFORM_OS_CROS:
      return "cros";
    case PLATFORM_OS_LINUX:
      return "linux";
    case PLATFORM_OS_OPENBSD:
      return "openbsd";
    case PLATFORM_OS_FUCHSIA:
      return "fuchsia";
    case PLATFORM_OS_NONE:
      return "";
  }
  NOTREACHED();
  return "";
}

PlatformOs ParsePlatformOs(const std::string& enum_string) {
  if (enum_string == "mac")
    return PLATFORM_OS_MAC;
  if (enum_string == "win")
    return PLATFORM_OS_WIN;
  if (enum_string == "android")
    return PLATFORM_OS_ANDROID;
  if (enum_string == "cros")
    return PLATFORM_OS_CROS;
  if (enum_string == "linux")
    return PLATFORM_OS_LINUX;
  if (enum_string == "openbsd")
    return PLATFORM_OS_OPENBSD;
  if (enum_string == "fuchsia")
    return PLATFORM_OS_FUCHSIA;
  return PLATFORM_OS_NONE;
}


const char* ToString(PlatformArch enum_param) {
  switch (enum_param) {
    case PLATFORM_ARCH_ARM:
      return "arm";
    case PLATFORM_ARCH_ARM64:
      return "arm64";
    case PLATFORM_ARCH_X86_32:
      return "x86-32";
    case PLATFORM_ARCH_X86_64:
      return "x86-64";
    case PLATFORM_ARCH_MIPS:
      return "mips";
    case PLATFORM_ARCH_MIPS64:
      return "mips64";
    case PLATFORM_ARCH_NONE:
      return "";
  }
  NOTREACHED();
  return "";
}

PlatformArch ParsePlatformArch(const std::string& enum_string) {
  if (enum_string == "arm")
    return PLATFORM_ARCH_ARM;
  if (enum_string == "arm64")
    return PLATFORM_ARCH_ARM64;
  if (enum_string == "x86-32")
    return PLATFORM_ARCH_X86_32;
  if (enum_string == "x86-64")
    return PLATFORM_ARCH_X86_64;
  if (enum_string == "mips")
    return PLATFORM_ARCH_MIPS;
  if (enum_string == "mips64")
    return PLATFORM_ARCH_MIPS64;
  return PLATFORM_ARCH_NONE;
}


const char* ToString(PlatformNaclArch enum_param) {
  switch (enum_param) {
    case PLATFORM_NACL_ARCH_ARM:
      return "arm";
    case PLATFORM_NACL_ARCH_X86_32:
      return "x86-32";
    case PLATFORM_NACL_ARCH_X86_64:
      return "x86-64";
    case PLATFORM_NACL_ARCH_MIPS:
      return "mips";
    case PLATFORM_NACL_ARCH_MIPS64:
      return "mips64";
    case PLATFORM_NACL_ARCH_NONE:
      return "";
  }
  NOTREACHED();
  return "";
}

PlatformNaclArch ParsePlatformNaclArch(const std::string& enum_string) {
  if (enum_string == "arm")
    return PLATFORM_NACL_ARCH_ARM;
  if (enum_string == "x86-32")
    return PLATFORM_NACL_ARCH_X86_32;
  if (enum_string == "x86-64")
    return PLATFORM_NACL_ARCH_X86_64;
  if (enum_string == "mips")
    return PLATFORM_NACL_ARCH_MIPS;
  if (enum_string == "mips64")
    return PLATFORM_NACL_ARCH_MIPS64;
  return PLATFORM_NACL_ARCH_NONE;
}


PlatformInfo::PlatformInfo()
: os(PLATFORM_OS_NONE),
arch(PLATFORM_ARCH_NONE),
nacl_arch(PLATFORM_NACL_ARCH_NONE) {}

PlatformInfo::~PlatformInfo() = default;
PlatformInfo::PlatformInfo(PlatformInfo&& rhs) = default;
PlatformInfo& PlatformInfo::operator=(PlatformInfo&& rhs) = default;
// static
bool PlatformInfo::Populate(
    const base::Value& value, PlatformInfo* out) {
  if (!value.is_dict()) {
    return false;
  }
  const base::Value::Dict& dict = value.GetDict();
  const base::Value* os_value = dict.Find("os");
  if (!os_value) {
    return false;
  }
  {
    const std::string* platform_os_as_string = (*os_value).GetIfString();
    if (!platform_os_as_string) {
      return false;
    }
    out->os = ParsePlatformOs(*platform_os_as_string);
    if (out->os == PLATFORM_OS_NONE) {
      return false;
    }
  }

  const base::Value* arch_value = dict.Find("arch");
  if (!arch_value) {
    return false;
  }
  {
    const std::string* platform_arch_as_string = (*arch_value).GetIfString();
    if (!platform_arch_as_string) {
      return false;
    }
    out->arch = ParsePlatformArch(*platform_arch_as_string);
    if (out->arch == PLATFORM_ARCH_NONE) {
      return false;
    }
  }

  const base::Value* nacl_arch_value = dict.Find("nacl_arch");
  if (!nacl_arch_value) {
    return false;
  }
  {
    const std::string* platform_nacl_arch_as_string = (*nacl_arch_value).GetIfString();
    if (!platform_nacl_arch_as_string) {
      return false;
    }
    out->nacl_arch = ParsePlatformNaclArch(*platform_nacl_arch_as_string);
    if (out->nacl_arch == PLATFORM_NACL_ARCH_NONE) {
      return false;
    }
  }

  return true;
}

// static
std::unique_ptr<PlatformInfo> PlatformInfo::FromValue(const base::Value& value) {
  auto out = std::make_unique<PlatformInfo>();
  bool result = Populate(value, out.get());
  if (!result)
    return nullptr;
  return out;
}

base::Value::Dict PlatformInfo::ToValue() const {
  base::Value::Dict to_value_result;

  to_value_result.Set("os", runtime::ToString(this->os));

  to_value_result.Set("arch", runtime::ToString(this->arch));

  to_value_result.Set("nacl_arch", runtime::ToString(this->nacl_arch));


  return to_value_result;
}


const char* ToString(RequestUpdateCheckStatus enum_param) {
  switch (enum_param) {
    case REQUEST_UPDATE_CHECK_STATUS_THROTTLED:
      return "throttled";
    case REQUEST_UPDATE_CHECK_STATUS_NO_UPDATE:
      return "no_update";
    case REQUEST_UPDATE_CHECK_STATUS_UPDATE_AVAILABLE:
      return "update_available";
    case REQUEST_UPDATE_CHECK_STATUS_NONE:
      return "";
  }
  NOTREACHED();
  return "";
}

RequestUpdateCheckStatus ParseRequestUpdateCheckStatus(const std::string& enum_string) {
  if (enum_string == "throttled")
    return REQUEST_UPDATE_CHECK_STATUS_THROTTLED;
  if (enum_string == "no_update")
    return REQUEST_UPDATE_CHECK_STATUS_NO_UPDATE;
  if (enum_string == "update_available")
    return REQUEST_UPDATE_CHECK_STATUS_UPDATE_AVAILABLE;
  return REQUEST_UPDATE_CHECK_STATUS_NONE;
}


const char* ToString(OnInstalledReason enum_param) {
  switch (enum_param) {
    case ON_INSTALLED_REASON_INSTALL:
      return "install";
    case ON_INSTALLED_REASON_UPDATE:
      return "update";
    case ON_INSTALLED_REASON_CHROME_UPDATE:
      return "chrome_update";
    case ON_INSTALLED_REASON_SHARED_MODULE_UPDATE:
      return "shared_module_update";
    case ON_INSTALLED_REASON_NONE:
      return "";
  }
  NOTREACHED();
  return "";
}

OnInstalledReason ParseOnInstalledReason(const std::string& enum_string) {
  if (enum_string == "install")
    return ON_INSTALLED_REASON_INSTALL;
  if (enum_string == "update")
    return ON_INSTALLED_REASON_UPDATE;
  if (enum_string == "chrome_update")
    return ON_INSTALLED_REASON_CHROME_UPDATE;
  if (enum_string == "shared_module_update")
    return ON_INSTALLED_REASON_SHARED_MODULE_UPDATE;
  return ON_INSTALLED_REASON_NONE;
}


const char* ToString(OnRestartRequiredReason enum_param) {
  switch (enum_param) {
    case ON_RESTART_REQUIRED_REASON_APP_UPDATE:
      return "app_update";
    case ON_RESTART_REQUIRED_REASON_OS_UPDATE:
      return "os_update";
    case ON_RESTART_REQUIRED_REASON_PERIODIC:
      return "periodic";
    case ON_RESTART_REQUIRED_REASON_NONE:
      return "";
  }
  NOTREACHED();
  return "";
}

OnRestartRequiredReason ParseOnRestartRequiredReason(const std::string& enum_string) {
  if (enum_string == "app_update")
    return ON_RESTART_REQUIRED_REASON_APP_UPDATE;
  if (enum_string == "os_update")
    return ON_RESTART_REQUIRED_REASON_OS_UPDATE;
  if (enum_string == "periodic")
    return ON_RESTART_REQUIRED_REASON_PERIODIC;
  return ON_RESTART_REQUIRED_REASON_NONE;
}



//
// Functions
//

namespace GetBackgroundPage {

Results::BackgroundPage::BackgroundPage()
 {}

Results::BackgroundPage::~BackgroundPage() = default;
Results::BackgroundPage::BackgroundPage(BackgroundPage&& rhs) = default;
Results::BackgroundPage& Results::BackgroundPage::operator=(BackgroundPage&& rhs) = default;
base::Value::Dict Results::BackgroundPage::ToValue() const {
  base::Value::Dict to_value_result;

  to_value_result.Merge(additional_properties.Clone());

  return to_value_result;
}


base::Value::List Results::Create(const BackgroundPage& background_page) {
  base::Value::List create_results;
  create_results.reserve(1);
  create_results.Append((background_page).ToValue());

  return create_results;
}
}  // namespace GetBackgroundPage

namespace OpenOptionsPage {

base::Value::List Results::Create() {
  base::Value::List create_results;

  return create_results;
}
}  // namespace OpenOptionsPage

namespace SetUninstallURL {

Params::Params() = default;
Params::~Params() = default;

// static
std::unique_ptr<Params> Params::Create(const base::Value::List& args) {
  if (args.size() != 1) {
    return nullptr;
  }
  std::unique_ptr<Params> params(new Params());

  if (0 < args.size() &&
      !args[0].is_none()) {
    const base::Value& url_value = args[0];
    {
      auto* temp = url_value.GetIfString();
      if (!temp) {
        return std::unique_ptr<Params>();
      }
      params->url = *temp;
    }
  }
  else {
    return std::unique_ptr<Params>();
  }

  return params;
}


base::Value::List Results::Create() {
  base::Value::List create_results;

  return create_results;
}
}  // namespace SetUninstallURL

namespace Reload {

}  // namespace Reload

namespace RequestUpdateCheck {

Results::Result::Result()
: status(REQUEST_UPDATE_CHECK_STATUS_NONE) {}

Results::Result::~Result() = default;
Results::Result::Result(Result&& rhs) = default;
Results::Result& Results::Result::operator=(Result&& rhs) = default;
base::Value::Dict Results::Result::ToValue() const {
  base::Value::Dict to_value_result;

  to_value_result.Set("status", runtime::ToString(this->status));

  if (this->version) {
    to_value_result.Set("version", *this->version);

  }

  return to_value_result;
}


base::Value::List Results::Create(const Result& result) {
  base::Value::List create_results;
  create_results.reserve(1);
  create_results.Append((result).ToValue());

  return create_results;
}
}  // namespace RequestUpdateCheck

namespace Restart {

}  // namespace Restart

namespace RestartAfterDelay {

Params::Params() = default;
Params::~Params() = default;

// static
std::unique_ptr<Params> Params::Create(const base::Value::List& args) {
  if (args.size() != 1) {
    return nullptr;
  }
  std::unique_ptr<Params> params(new Params());

  if (0 < args.size() &&
      !args[0].is_none()) {
    const base::Value& seconds_value = args[0];
    {
      auto temp = seconds_value.GetIfInt();
      if (!temp.has_value()) {
        return std::unique_ptr<Params>();
      }
      params->seconds = *temp;
    }
  }
  else {
    return std::unique_ptr<Params>();
  }

  return params;
}


base::Value::List Results::Create() {
  base::Value::List create_results;

  return create_results;
}
}  // namespace RestartAfterDelay

namespace GetPlatformInfo {

base::Value::List Results::Create(const PlatformInfo& platform_info) {
  base::Value::List create_results;
  create_results.reserve(1);
  create_results.Append((platform_info).ToValue());

  return create_results;
}
}  // namespace GetPlatformInfo

namespace GetPackageDirectoryEntry {

Results::DirectoryEntry::DirectoryEntry()
 {}

Results::DirectoryEntry::~DirectoryEntry() = default;
Results::DirectoryEntry::DirectoryEntry(DirectoryEntry&& rhs) = default;
Results::DirectoryEntry& Results::DirectoryEntry::operator=(DirectoryEntry&& rhs) = default;
base::Value::Dict Results::DirectoryEntry::ToValue() const {
  base::Value::Dict to_value_result;

  to_value_result.Merge(additional_properties.Clone());

  return to_value_result;
}


base::Value::List Results::Create(const DirectoryEntry& directory_entry) {
  base::Value::List create_results;
  create_results.reserve(1);
  create_results.Append((directory_entry).ToValue());

  return create_results;
}
}  // namespace GetPackageDirectoryEntry

//
// Events
//

namespace OnStartup {

const char kEventName[] = "runtime.onStartup";

base::Value::List Create() {
  base::Value::List create_results;

  return create_results;
}

}  // namespace OnStartup

namespace OnInstalled {

const char kEventName[] = "runtime.onInstalled";

Details::Details()
: reason(ON_INSTALLED_REASON_NONE) {}

Details::~Details() = default;
Details::Details(Details&& rhs) = default;
Details& Details::operator=(Details&& rhs) = default;
base::Value::Dict Details::ToValue() const {
  base::Value::Dict to_value_result;

  to_value_result.Set("reason", runtime::ToString(this->reason));

  if (this->previous_version) {
    to_value_result.Set("previousVersion", *this->previous_version);

  }
  if (this->id) {
    to_value_result.Set("id", *this->id);

  }

  return to_value_result;
}


base::Value::List Create(const Details& details) {
  base::Value::List create_results;
  create_results.reserve(1);
  create_results.Append((details).ToValue());

  return create_results;
}

}  // namespace OnInstalled

namespace OnSuspend {

const char kEventName[] = "runtime.onSuspend";

base::Value::List Create() {
  base::Value::List create_results;

  return create_results;
}

}  // namespace OnSuspend

namespace OnSuspendCanceled {

const char kEventName[] = "runtime.onSuspendCanceled";

base::Value::List Create() {
  base::Value::List create_results;

  return create_results;
}

}  // namespace OnSuspendCanceled

namespace OnUpdateAvailable {

const char kEventName[] = "runtime.onUpdateAvailable";

Details::Details()
 {}

Details::~Details() = default;
Details::Details(Details&& rhs) = default;
Details& Details::operator=(Details&& rhs) = default;
base::Value::Dict Details::ToValue() const {
  base::Value::Dict to_value_result;

  to_value_result.Set("version", this->version);

  to_value_result.Merge(additional_properties.Clone());

  return to_value_result;
}


base::Value::List Create(const Details& details) {
  base::Value::List create_results;
  create_results.reserve(1);
  create_results.Append((details).ToValue());

  return create_results;
}

}  // namespace OnUpdateAvailable

namespace OnBrowserUpdateAvailable {

const char kEventName[] = "runtime.onBrowserUpdateAvailable";

base::Value::List Create() {
  base::Value::List create_results;

  return create_results;
}

}  // namespace OnBrowserUpdateAvailable

namespace OnRestartRequired {

const char kEventName[] = "runtime.onRestartRequired";

base::Value::List Create(const OnRestartRequiredReason& reason) {
  base::Value::List create_results;
  create_results.reserve(1);
  create_results.Append(runtime::ToString(reason));

  return create_results;
}

}  // namespace OnRestartRequired

}  // namespace runtime
}  // namespace api
}  // namespace extensions

