// Copyright 2024 The Chromium Authors
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// GENERATED FROM THE API DEFINITION IN
//   qtwebengine/common/extensions/api/webrtc_desktop_capture_private.idl
// by tools/json_schema_compiler.
// DO NOT EDIT.

#include "qtwebengine/common/extensions/api/webrtc_desktop_capture_private.h"

#include <memory>
#include <ostream>
#include <string>
#include <utility>
#include <vector>

#include "base/check.h"
#include "base/check_op.h"
#include "base/notreached.h"
#include "base/strings/string_number_conversions.h"
#include "base/strings/utf_string_conversions.h"
#include "base/values.h"
#include "tools/json_schema_compiler/util.h"

using base::UTF8ToUTF16;

namespace extensions {
namespace api {
namespace webrtc_desktop_capture_private {
//
// Types
//

RequestInfo::RequestInfo()
: guest_process_id(0),
guest_render_frame_id(0) {}

RequestInfo::~RequestInfo() = default;
RequestInfo::RequestInfo(RequestInfo&& rhs) = default;
RequestInfo& RequestInfo::operator=(RequestInfo&& rhs) = default;
// static
bool RequestInfo::Populate(
    const base::Value& value, RequestInfo* out) {
  if (!value.is_dict()) {
    return false;
  }
  const base::Value::Dict& dict = value.GetDict();
  const base::Value* guest_process_id_value = dict.Find("guestProcessId");
  if (!guest_process_id_value) {
    return false;
  }
  {
    auto temp = (*guest_process_id_value).GetIfInt();
    if (!temp.has_value()) {
      return false;
    }
    out->guest_process_id = *temp;
  }

  const base::Value* guest_render_frame_id_value = dict.Find("guestRenderFrameId");
  if (!guest_render_frame_id_value) {
    return false;
  }
  {
    auto temp = (*guest_render_frame_id_value).GetIfInt();
    if (!temp.has_value()) {
      return false;
    }
    out->guest_render_frame_id = *temp;
  }

  return true;
}

// static
std::unique_ptr<RequestInfo> RequestInfo::FromValue(const base::Value& value) {
  auto out = std::make_unique<RequestInfo>();
  bool result = Populate(value, out.get());
  if (!result)
    return nullptr;
  return out;
}

base::Value::Dict RequestInfo::ToValue() const {
  base::Value::Dict to_value_result;

  to_value_result.Set("guestProcessId", this->guest_process_id);

  to_value_result.Set("guestRenderFrameId", this->guest_render_frame_id);


  return to_value_result;
}


Options::Options()
: can_request_audio_track(false) {}

Options::~Options() = default;
Options::Options(Options&& rhs) = default;
Options& Options::operator=(Options&& rhs) = default;
// static
bool Options::Populate(
    const base::Value& value, Options* out) {
  if (!value.is_dict()) {
    return false;
  }
  const base::Value::Dict& dict = value.GetDict();
  const base::Value* can_request_audio_track_value = dict.Find("canRequestAudioTrack");
  if (!can_request_audio_track_value) {
    return false;
  }
  {
    auto temp = (*can_request_audio_track_value).GetIfBool();
    if (!temp.has_value()) {
      return false;
    }
    out->can_request_audio_track = *temp;
  }

  return true;
}

// static
std::unique_ptr<Options> Options::FromValue(const base::Value& value) {
  auto out = std::make_unique<Options>();
  bool result = Populate(value, out.get());
  if (!result)
    return nullptr;
  return out;
}

base::Value::Dict Options::ToValue() const {
  base::Value::Dict to_value_result;

  to_value_result.Set("canRequestAudioTrack", this->can_request_audio_track);


  return to_value_result;
}



//
// Functions
//

namespace ChooseDesktopMedia {

Params::Params() = default;
Params::~Params() = default;

// static
std::unique_ptr<Params> Params::Create(const base::Value::List& args) {
  if (args.size() != 1) {
    return nullptr;
  }
  std::unique_ptr<Params> params(new Params());

  if (0 < args.size() &&
      !args[0].is_none()) {
    const base::Value& request_value = args[0];
    {
      if (!request_value.is_dict()) {
        return std::unique_ptr<Params>();
      }
      if (!RequestInfo::Populate(request_value, &params->request)) {
        return std::unique_ptr<Params>();
      }
    }
  }
  else {
    return std::unique_ptr<Params>();
  }

  return params;
}


base::Value::List Results::Create(const std::string& stream_id, const Options& options) {
  base::Value::List create_results;
  create_results.reserve(2);
  create_results.Append(stream_id);

  create_results.Append((options).ToValue());

  return create_results;
}
}  // namespace ChooseDesktopMedia

}  // namespace webrtc_desktop_capture_private
}  // namespace api
}  // namespace extensions

